/***************************************************************************
 *  SEABAS Control base class "MySiTCP" (inherit SiTCP_Cnt)
 *
 *  File        : MySiTCP.cc 
 *  version     : v 1.0.1 2008/11/26 
 *  Author      : Minoru Hirose
 *
 *  Description : class MySiTCP Methods 
 *
 ****************************************************************************/
#include "MySiTCP.h"

void MySiTCP::DAQStart(bool start){
  RBCPskeleton(0xff, 0x80, 0x0, 0x1, 0x0);

  sndData[0] = start ? 1 : 0;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);

  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
 (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAQ Start Signal");
    exit(1);
  }
  //  if(start)std::cout<<"Send DAQ Start Signal to SiTCP Board... Done"<<std::endl;
  //  else std::cout<<"Send DAQ Stop Signal to SiTCP Board... Done"<<std::endl;
  rcvRBCP_ACK(0);
}

void MySiTCP::NumOfTakeData(int num){
  if(num<0 || 256<num){
    std::cout<<"# of Taking Data should be 0-255"<<std::endl;
    exit(1);
  }
  RBCPskeleton(0xff, 0x80, 0x1, 0x1, 0x1);
 
  sndData[0] = num;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);

  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set #event of taking data");
    exit(1);
  }
  puts("");
  //  if(num==0)std::cout<<"# of Taking Data == endless"<<std::endl;
  //  else std::cout<<"# of Taking Data == "<<num<<std::endl;
  rcvRBCP_ACK(0);
}

void MySiTCP::SetRSTHiTime(int time){
  if(time<0 || 255<time){
    std::cout<<"RST Hi Time should be 0-255*40ns"<<std::endl;
    exit(1);
  }
  RBCPskeleton(0xff, 0x80, 0x5, 0x1, 0x5);
  sndData[0] = time;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);

  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set RST interbal");
    exit(1);
  }
  rcvRBCP_ACK(0);
}

void MySiTCP::SetNIM_CLK(int num){
  if(num<0 || 9<num){
    std::cout<<"NIM_CLK set number should be 0-8"<<std::endl;
    std::cout<<"0 -> 1.5Hz"<<std::endl;
    std::cout<<"1 ->   3Hz"<<std::endl;
    std::cout<<"2 ->   6Hz"<<std::endl;
    std::cout<<"3 ->  12Hz"<<std::endl;
    std::cout<<"4 ->  24Hz"<<std::endl;
    std::cout<<"5 ->  48Hz"<<std::endl;
    std::cout<<"6 ->  95Hz"<<std::endl;
    std::cout<<"7 -> 191Hz"<<std::endl;
    std::cout<<"8 ->   0Hz"<<std::endl;
    exit(1);
  }
  RBCPskeleton(0xff, 0x80, 0x2, 0x1, 0x2);
  sndData[0] = num;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);

  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set NIM_CLK number");
    exit(1);
  }
  rcvRBCP_ACK(0);
}

void MySiTCP::SetPixScanTime(int time){
  if(time<0 || 16<time){
    std::cout<<"pix scan time set number should be 0-15"<<std::endl;
    std::cout<<"pixel scanning every (num+1)*40ns"<<std::endl;
    exit(1);
  }
  RBCPskeleton(0xff, 0x80, 0x9, 0x1, 0x9);
  sndData[0] = time-1;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);

  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0, (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Set pixel scan interbal");
    exit(1);
  }
  rcvRBCP_ACK(0);
}

void MySiTCP::SetRSTInterbal(int time){
  if(!(300<=time && time<=6000000 || time==0)){
    std::cout<<"RST Interbal value should be 300-6000000*40ns"<<std::endl;
    std::cout<<time<<" isn't valid value"<<std::endl;
    exit(1);
  }
  char addr6, addr7, addr8;
  addr8 = (char){((unsigned int)time & 0xFF0000) >> 16};
  addr7 = (char){((unsigned int)time & 0x00FF00) >> 8};
  addr6 = (char){((unsigned int)time & 0x0000FF) >> 0};
  //  std::cout<<(unsigned short)addr6<<std::endl;
  //  std::cout<<(unsigned short)addr7<<std::endl;
  //  std::cout<<(unsigned short)addr8<<std::endl;
  usleep(50);

  //send addr6 signal
  RBCPskeleton(0xff, 0x80, 0x6, 0x1, 0x6);
  sndData[0] = time ? (unsigned short)addr6 : 0;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);
  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send RST Interbal (ADDR[6])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(50);

  //send addr7 signal
   RBCPskeleton(0xff, 0x80, 0x7, 0x1, 0x7);
   sndData[0] = time ? (unsigned short)addr7 : 0;
   memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);
   len = 0;
   len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send RST Interbal (ADDR[7])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);

  //send addr8 signal
   RBCPskeleton(0xff, 0x80, 0x8, 0x1, 0x8);
   sndData[0] = time ? (unsigned short)addr8 : 0;
   memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);
   len = 0;
   len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send RST Interbal (ADDR[8])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);
    
} 

int MySiTCP::DAC_cnt(int ch, int mV){
  if(!(0<=mV && mV<=3000)){
    std::cout<<"DAC output should be 0-3000[mV]"<<std::endl;
    std::cout<<mV<<"[mV] isn't valid value"<<std::endl;
    std::cout<<"Now, No output!"<<std::endl;
    return -1;
  }
  if(!(0<=ch && ch<=3)){
    std::cout<<"DAC ch is 0-3"<<std::endl;
    std::cout<<ch<<" isn't valid value"<<std::endl;
  }
  char addr3, addr4;
  double bit;
  bit = (double)mV*4095/3300*1.1;//1.1 is unknown factor to calibrate
  addr3 = (char){(unsigned short)floor(bit) >> 8};
  addr4 = (char){(unsigned short)floor(bit) & 0x00FF};
  addr3 = addr3 + (ch<<6);
  //  std::cout<<(unsigned short)addr3<<std::endl;
  //std::cout<<(unsigned short)addr4<<std::endl;
  usleep(50);

  //send addr3 signal
  RBCPskeleton(0xff, 0x80, 0x3, 0x1, 0x3);
  sndData[0] = (unsigned short)addr3;
  memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);
  int len = 0;
  len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
  if(len < 0){
    perror("Send DAC Control Signal (ADDR[3])");
    exit(1);
  }
  rcvRBCP_ACK(0);
  usleep(50);

  //send addr4 signal
   RBCPskeleton(0xff, 0x80, 0x4, 0x1, 0x4);
   sndData[0] = (unsigned short)addr4;
   memcpy(sndBuf+sizeof(GetsndHeader()), sndData, 1);
   len = 0;
   len = sendto(GetUDPSock(), sndBuf, 1 + sizeof(GetsndHeader()), 0,
 	       (struct sockaddr *)&udpAddr, sizeof(udpAddr));
   if(len < 0){
     perror("Send DAC Control Signal (ADDR[4])");
     exit(1);
   }
   rcvRBCP_ACK(0);
   usleep(50);
    
  return 0;
} 

int MySiTCP::TakeDatAt(unsigned short *data){
  bool ret = true;
  int rlen = 0;
  rlen = recv(GetTCPSock(), &tmp[0][0], sizeof(tmp), MSG_WAITALL);
  //rlen = recv(GetTCPSock(), &tmp[0][0], sizeof(tmp), 0);                                                 
  //std::cout<<"receive length = "<<rlen<<std::endl;                                                  
  if(rlen < 0){
    perror("receive");
    ret = false;
    //    exit(1);                                                                                    
  }else if(rlen != 32768){
    std::cout<<"receive failuer : rlen = "<<rlen<<std::endl;
  }
  memcpy(data, tmp, sizeof(tmp));
  return rlen;
}

int MySiTCP::tcpselect(int usec){
  //return 0 : There is a keyboard input .    
  //return 1 : tcp receiving data ready                          
  //return 2 : timeout
  //return -1 : ERROR         
  memcpy(&fds, &readfds, sizeof(fd_set));
  timeout.tv_sec = 0;
  timeout.tv_usec = usec;
  int n = select(GetTCPSock()+1, &fds, NULL, NULL, &timeout);
  if(n==0)return 2;
  //select(GetTCPSock()+1, &fds, NULL, NULL, NULL);

  if(FD_ISSET(GetTCPSock(), &fds)){
    return 1;
  }else if(FD_ISSET(0, &fds)){
    return 0;
  }else{
    return -1;
  }
}


