/***************************************************************************
*  INTPIX DAQ Program
*  
*  File        : mymainframe.cc
*  version     : v 1.0.0 2008/11/5
*  Author      : Minoru Hirose
*
*  Description : class MyMainFrame  Methods
*                class MyCanvas     Methods                     
*                class MyOption     Methods                     
*                class MyParameters Methods                     
*                class MylogEdit    Methods                     
*
*  Original Program is made for COBI-INTPIX system by Shun Ono.
*  I appreciate all his work.
***************************************************************************/

#include "mymainframe.h"

MyMainFrame::MyMainFrame(const TGWindow *p,UInt_t w,UInt_t h) : TGMainFrame(p,w,h)
{
  gStyle->SetPalette(1,0);
  gROOT->SetStyle("Plain");
  
  f1 = new TF1("f1","-x",0,128);
  revAxis = new TGaxis(0,0,0,128,"f1",510,"-");

  h2[0] = new TH2F("h20","2D hist (Raw data)",128,0,128,128,0,128);
  h2[1] = new TH2F("h21","2D hist (Raw data - Ped)",128,0,128,128,0,128);
  h2[2] = new TH2F("h22","2D hist (Raw data - Ped - CMS)",128,0,128,128,0,128);
  h2[3] = new TH2F("h23","2D hist (SNR)",128,0,128,128,0,128);
  h2[4] = new TH2F("h24","2D hist (Ped)",128,0,128,128,0,128);
  h2[5] = new TH2F("h25","2D hist (Noise)",128,0,128,128,0,128);
  h2[6] = new TH2F("h26","2D hist (Bad channel)",128,0,128,128,0,128);
  for(int i=0; i<7 ; i++){
    h2[i]->SetXTitle("CA");
    h2[i]->SetYTitle("RA");
    h2[i]->GetYaxis()->SetLabelSize(0);
    h2[i]->GetYaxis()->SetTickLength(0);
  }

  h1[0] = new TH1F("h10","1D hist (Raw data)",(4096-8)/4,4,4092);
  h1[1] = new TH1F("h11","1D hist (Raw data - Ped)",8100,-4000,4100);
  h1[2] = new TH1F("h12","1D hist (Raw data - Ped - CMS)",8100,-4000,4100);
  h1[3] = new TH1F("h13","1D hist (SNR)",100,0,200);
  h1[4] = new TH1F("h14","1D hist (Ped)",(4096-8)/4,4,4092);
  h1[5] = new TH1F("h15","1D hist (Noise)",100,0,20);
  h1[6] = new TH1F("h16","1D hist (Bad channel)",3,0,3);

  utime = new TTimer();  
  dtime = new TTimer();
  
  cal = new Calib();
  //MyMenu = new MyMenuBar(this);
  //MyMenu->MenuLayout();
  //AddFrame(MyMenu, new TGLayoutHints(kLHintsTop | kLHintsExpandX));

  //Main horizontal frame
  MainHorzFrame = new TGHorizontalFrame(this,700,500);
  Canv = new MyCanvas(MainHorzFrame,500,500);
  Canv->CanvLayout();
  MainHorzFrame->AddFrame(Canv, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 5,5,3,1));
 
  Opt = new MyOption(MainHorzFrame,175,500);
  Opt->OptLayout();
  MainHorzFrame->AddFrame(Opt, new TGLayoutHints(kLHintsLeft,5,5,7,4));

  Para = new MyParameters(MainHorzFrame,175,500);
  Para->ParaLayout();
  MainHorzFrame->AddFrame(Para, new TGLayoutHints(kLHintsLeft,5,5,7,4));
 
  Cont = new MyCtrl(MainHorzFrame,175,500);
  Cont->CtrlLayout();
  MainHorzFrame->AddFrame(Cont, new TGLayoutHints(kLHintsLeft,5,5,8,4)); 
  AddFrame(MainHorzFrame, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,2,2,2,2));
  
  //Sub frame
  SubHorzFrame = new TGHorizontalFrame(this,900,200); 
  Log =  new MylogEdit(SubHorzFrame, new TGString("Control log"));
  Log->EdtLayout();
  SubHorzFrame->AddFrame(Log, new TGLayoutHints(kLHintsLeft | kLHintsExpandY ,5,5,7,7)); 

  exit = new TGTextButton(SubHorzFrame,"&Exit");
  SubHorzFrame->AddFrame(exit, new TGLayoutHints(kLHintsRight | kLHintsBottom,5,5,7,7));
  reset = new TGTextButton(SubHorzFrame,"&Reset");
  SubHorzFrame->AddFrame(reset, new TGLayoutHints(kLHintsRight | kLHintsBottom,5,5,7,7));  
  AddFrame(SubHorzFrame, new TGLayoutHints(kLHintsExpandX,2,2,2,2));

  Initialize();
  Connection();  
  SetStatus("Starting INTPIX DAQ Program.",1);
  SetStatus("Author : Minoru Hirose & Shun Ono",1);
  SetStatus("",1);

  sitcp = new MySiTCP();
  SetStatus("Start SEABAS Operation",1);
  sitcp->SetIPPort("192.168.0.16", 24, 4660);
  int tcpret, udpret;
  udpret = sitcp->CreateUDPSock();
  tcpret = sitcp->CreateTCPSock();
  int retval;
  if(udpret<0){
    wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Cannot create UDP Socket.",kMBIconExclamation,kMBOk,&retval);
  }    
  switch(tcpret){
  case 1:
    SetStatus("Connecting to SEABAS : Success!",1);
    break;
  case -1:
    wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Cannot create TCP Socket.",kMBIconExclamation,kMBOk,&retval);
    break;
  case -2:
    wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","TCP Connection Timeout",kMBIconExclamation,kMBOk,&retval);
    SetStatus("***Check your network configuration or SEABAS power supply***",1);
    break;
  case -3:
    wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","TCP Connection Error",kMBIconExclamation,kMBOk,&retval);
    SetStatus("***Check your network configuration or SEABAS power supply***",1);
   break;
  }
  if(udpret<0 || tcpret<0){
    Cont->GetStart()->SetEnabled(kFALSE);
  }
  sitcp->SetRSTInterbal((int)(Para->GetIntTime()->GetNumber()*1000/40));
  sitcp->SetRSTHiTime((int)(Para->GetRSTSig()->GetNumber()/40));
  sitcp->SetPixScanTime((int)(Para->GetPixScan()->GetNumber()/40));
  sitcp->DAC_cnt(0,(int)(Para->GetRSTV()->GetNumber()));

  SetWindowName("DAQ program for INTPIX2");
  MapSubwindows();
  Resize(GetDefaultSize());
  MapWindow();
}

void MyMainFrame::Initialize()
{
  Canv->GetECanvas()->GetCanvas()->SetRightMargin(0.13);
  Canv->GetECanvas()->GetCanvas()->Clear();
  Canv->GetECanvas()->GetCanvas()->Update();
  Canv->GetUpdInt()->SetNumber(1.00);
  Cont->GetNevent()->SetNumber(0);
  Cont->GetCevent()->SetNumber(0);
  Opt->GetCalib()->SetNumber(200);
  Opt->GetHistHi()->SetNumber(4096);
  Opt->GetHistLow()->SetNumber(0);
  Para->GetIntTime()->SetNumber(100);
  Para->GetRSTSig()->SetNumber(2000);
  Para->GetPixScan()->SetNumber(320);
  Para->GetRSTV()->SetNumber(700);
  
  Cont->GetStop()->SetEnabled(kFALSE);
  Cont->GetPause()->SetEnabled(kFALSE);
  fildirpath = getenv("PWD");
  Cont->GetDEntry()->SetText(GetRltPath(fildirpath));

  utime->SetTime( (int)(Canv->GetUpdInt()->GetNumber()*1000));
  dtime->SetTime(10);

  gStyle->SetOptStat("eMinoru");
  //  gStyle->SetStatFontSize(0.08);
  gStyle->SetStatH(0.10);
  gStyle->SetStatW(0.30);
  beforeRSTSig=2000;
  beforePixScan=320;
  pauseflg=0;
  calibflag=0;
  runmode=2;
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      data[i][j]=0;
      raw[i][j]=0;
      bch[i][j]=0;
      ped[i][j]=0;
      ino[i][j]=1;
    }
}

void MyMainFrame::Connection()
{
  Canv->GetDAQIntCBox()->Connect("Selected(Int_t)","MyMainFrame",this,"SetDAQTime()");
  Canv->GetUpdInt()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetUpdateTime()");
  Opt->GetDispOpt()->Connect("Selected(Int_t)","MyMainFrame",this,"UpdateTimer()");
  Opt->GetHistCont()->Connect("Selected(Int_t)","MyMainFrame",this,"UpdateTimer()");
  Opt->GetLogX()->Connect("Clicked()","MyMainFrame",this,"UpdateTimer()");
  Opt->GetLogY()->Connect("Clicked()","MyMainFrame",this,"UpdateTimer()");
  Opt->GetLogZ()->Connect("Clicked()","MyMainFrame",this,"UpdateTimer()");
  Opt->GetHistHi()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetHistRange()");
  Opt->GetHistLow()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetHistRange()");
  Opt->GetRunMode()->Connect("Clicked(Int_t)","MyMainFrame",this,"SetRunMode()");
  Para->GetIntTime()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetIntTime()");
  Para->GetRSTSig()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetRSTSig()");
  Para->GetPixScan()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetPixScan()");
  Para->GetRSTV()->Connect("ValueSet(Long_t)","MyMainFrame",this,"SetRSTV()");
  Cont->GetStart()->Connect("Clicked()","MyMainFrame",this,"RunDAQ()");
  Cont->GetStop()->Connect("Clicked()","MyMainFrame",this,"StopDAQ()");
  Cont->GetPause()->Connect("Clicked()","MyMainFrame",this,"PauseDAQ()");
  Cont->GetFDir()->Connect("Clicked()","MyMainFrame",this,"DirFind()");
  Log->GetClrLog()->Connect("Clicked()","MyMainFrame",this,"ClearLogText()");
  Log->GetSaveLog()->Connect("Clicked()","MyMainFrame",this,"SaveLogText()");  
  reset->Connect("Clicked()","MyMainFrame",this,"Initialize()");
  exit->Connect("Clicked()","MyMainFrame",this,"CloseWindow()");
 
  utime->Connect("Timeout()", "MyMainFrame",this, "UpdateTimer()");  
  dtime->Connect("Timeout()", "MyMainFrame",this, "DAQTimer()");  
}

void MyMainFrame::DAQTimer(){
  int selectret = sitcp->tcpselect(100);
  if(selectret == 0){
    //    std::cout<<"interruptted DAQ"<<std::endl;
    //    std::cout<<"Exit"<<std::endl;
  }else if(selectret == 1){
    sitcp->TakeDatAt(&raw[0][0]);
    if(datafout){
      Intpix_Data wdata;
      if(calibflag==1){
	wdata.calib = true;
	wdata.event = curcalevent;
      }else{
	wdata.calib = false;
	wdata.event = curevent;
      }
      wdata.IntTime = (unsigned short)Para->GetIntTime()->GetNumber();
      wdata.RSTLength = (unsigned short)Para->GetRSTSig()->GetNumber();
      wdata.ScanTime = (unsigned short)Para->GetPixScan()->GetNumber();
      wdata.RSTV = (unsigned short)Para->GetRSTV()->GetNumber();
      memcpy(wdata.adc,&raw,sizeof(raw));
      
      datafout->write((char*)&wdata, sizeof(wdata));
    }
  
    if(calibflag==1){
      if(curcalevent<=pedevent-1){
	cal->AddPedData(raw);
	if(curcalevent==pedevent-1) cal->CalcPed();
      }else if(curcalevent<=noievent-1){
	cal->AddNoiData(raw);
	if(curcalevent==noievent-1) cal->CalcNoi();
      }
      
      curcalevent++;
      Cont->GetCevent()->SetIntNumber(curcalevent);
      if(curcalevent==totcalevent){
	calibflag=0;
	char logstat[1024];
	sprintf(logstat,"Calibration event end (%d event, pedestal %d event, noise %d event)",curcalevent, cal->PedEvent(), cal->NoiEvent());
	SetStatus(logstat,7);
	cal->OutPed(&ped[0][0]);
	cal->OutINoise(&ino[0][0]);
	cal->OutBChannel(&bch[0][0]);
	sitcp->SetNIM_CLK(1);
	
	if(runmode==1)StopDAQ();
	else{
	  PauseDAQ();
	}
      }
    }else{ //data
      curevent++; 
      Cont->GetCevent()->SetIntNumber(curevent);
      if(curevent%500 == 0){
	char logstat[1024];
	sprintf(logstat,"%d event were taken",curevent);
	SetStatus(logstat,7);
      }
      if(curevent==totevent){
	char logstat[1024];
	sprintf(logstat,"DAQ event end (%d event)",curevent);
	SetStatus(logstat,7);
	StopDAQ();
      }
    }
  }else if(selectret==2){
  }else{
    SetStatus("ERROR:select()",1);
    std::cout<<"ERROR:select()"<<std::endl;
  }

}

void MyMainFrame::UpdateTimer()
{

  int DisStr_Id = Opt->GetDispOpt()->GetSelected();
  int HstCnt_Id = Opt->GetHistCont()->GetSelected();
  
  if(DisStr_Id==0) h2[HstCnt_Id]->Reset();
  else if(DisStr_Id==1){
    h1[HstCnt_Id]->Reset();
  }
  double cms=0;
  if(HstCnt_Id==2 || HstCnt_Id==3){
    for(int i=0;i<128;i++)
      for(int j=0;j<128;j++){
	cms += ino[i][j];
      }
    cms /= (double)16384;
  }
  
  for(int i=0;i<128;i++)
    for(int j=0;j<128;j++){
      switch(HstCnt_Id){
      case 0:
	data[i][j] = raw[i][j];
	break;
      case 1:
	data[i][j] = (double)raw[i][j] - ped[i][j];
	if(data[i][j]<1) data[i][j]=1;
	break;
      case 2:
	data[i][j] = (double)raw[i][j] - ped[i][j] - cms;
	if(data[i][j]<1) data[i][j]=1;
	break;
      case 3:
	if(ino[i][j]!=0) data[i][j] = fabs((double)raw[i][j] - ped[i][j] - cms)/ino[i][j];
	else data[i][j] = 0;
	break;
      case 4:
	data[i][j] = ped[i][j];
	break;
      case 5:
	data[i][j] = ino[i][j];
	break;
      case 6:
	data[i][j] = bch[i][j];
      }
      if(DisStr_Id==0) h2[HstCnt_Id]->Fill(j,127-i,data[i][j]);
      else if(DisStr_Id==1) h1[HstCnt_Id]->Fill(data[i][j]);
    }

  if(DisStr_Id==0){
    h2[HstCnt_Id]->Draw("COLZ");
    //    h2[HstCnt_Id]->Draw("LEGO2");
    h2[HstCnt_Id]->SetStats(kFALSE);
    revAxis->Draw();
  }else if(DisStr_Id==1){
    h1[HstCnt_Id]->Draw();
  }
  

  TCanvas *fCanvas = Canv->GetECanvas()->GetCanvas(); 
  fCanvas->cd();
  if(DisStr_Id==1){
    if(Opt->GetLogX()->IsOn()) fCanvas->SetLogx(1);
    else fCanvas->SetLogx(0);
    if(Opt->GetLogY()->IsOn()) fCanvas->SetLogy(1); 
    else fCanvas->SetLogy(0);
    fCanvas->SetRightMargin(0.08);
  }else{
    fCanvas->SetLogx(0);
    fCanvas->SetLogy(0);
    if(Opt->GetLogZ()->IsOn()) fCanvas->SetLogz(1); 
    else fCanvas->SetLogz(0);
    fCanvas->SetRightMargin(0.13);
  }
  fCanvas->Modified();
  fCanvas->Update();

}

void MyMainFrame::RunDAQ()
{
  if(pauseflg==0){
    SetStatus("------------------------------------------------",1);
    totevent=(int)Cont->GetNevent()->GetNumber();
    curevent=0;
    calibflag=0;
    for(int i=0;i<128;i++)
      for(int j=0;j<128;j++){
	ped[i][j] = 0;
	ino[i][j] = 0;
	bch[i][j] = 0;
      }
  
    if(Cont->GetSavDat()->IsOn()){
      dataname = (char *)Cont->GetFEntry()->GetText();
      char dispstr[2048];
      int retval;
    
      if(strlen(dataname)==0){
	wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Save file name is empty",kMBIconExclamation,kMBOk,&retval);
	return;
      }
   
      sprintf(datafilepath, "%s/%s.dat", fildirpath, dataname);
      sprintf(logfilepath, "%s/%s.txt", fildirpath, dataname);
      ifstream fin;
      fin.open(logfilepath, ios::in);
      if(fin){
	fin.close();
	wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Warnig","Save file name already exist. Do you overwrite it?", kMBIconQuestion,kMBYes | kMBNo, &retval);
	if(retval==2) return;
      }
      //      datafout = new ofstream(datafilepath, ios::trunc | ios::binary);
      datafout = new ofstream(datafilepath, ios::trunc);
      logfout = new ofstream(logfilepath, ios::trunc);
      if(datafout->fail() || logfout->fail()){
	SetStatus("Error: Cannot open files",7);
	return;
      }
      sprintf(dispstr, "Save file name (File path = %s)", datafilepath);
      SetStatus(dispstr,7);     
    }
    if(runmode != 3){
      totcalevent=(int)Opt->GetCalib()->GetNumber();
      curcalevent=0;
      calibflag=1;
      cal->Reset();
      pedevent = totcalevent/2;
      noievent = totcalevent;
      char dispstr[2048];
      sprintf(dispstr, "Take calibration data (%d event)", totcalevent);
      SetStatus(dispstr,7);     
      sitcp->SetNIM_CLK(7);
    }else{
      int DAQInt_ID = Canv->GetDAQIntCBox()->GetSelected();
      sitcp->SetNIM_CLK(DAQInt_ID);
    }
    SetStatus("** Start run **",7);
    sitcp->DAQStart(true);
  }else{
    SetStatus("Restart run",7);
    int DAQInt_ID = Canv->GetDAQIntCBox()->GetSelected();
    sitcp->SetNIM_CLK(DAQInt_ID);
    sitcp->DAQStart(true);
  }
    
  if(calibflag==1 && curcalevent==0) SetStatus("Calibration event start",7);
  else if(calibflag==0 && curevent==0)  SetStatus("DAQ event start",7);

  Opt->GetCalib()->SetState(kFALSE);

  Cont->GetStart()->SetEnabled(kFALSE);
  Cont->GetStop()->SetEnabled(kTRUE);
  Cont->GetPause()->SetEnabled(kTRUE);
  Cont->GetNevent()->SetState(kFALSE);
  Cont->GetFEntry()->SetEnabled(kFALSE);
  Cont->GetSavDat()->SetEnabled(kFALSE);
  reset->SetEnabled(kFALSE);
  for(int i=1; i<=3; i++){
    Opt->GetRunMode()->GetButton(i)->SetEnabled(kFALSE);
  }

  pauseflg=0;

  utime->TurnOn();
  dtime->TurnOn();
}

void MyMainFrame::StopDAQ()
{  
  sitcp->DAQStart(false);
  utime->TurnOff();
  dtime->TurnOff();

  char dispstr[2048];
  sprintf(dispstr, "Total %d(calib:%d, data:%d) events were taken.", curcalevent+curevent, curcalevent, curevent);
  SetStatus(dispstr,7);
  SetStatus("** Stop run **",7);

  if(datafout && logfout){
    datafout->close();
    logfout->close();
    SetStatus("Close file",5);
  }

  Opt->GetCalib()->SetState(kTRUE);
 
  Cont->GetStart()->SetEnabled(kTRUE);
  Cont->GetStop()->SetEnabled(kFALSE);
  Cont->GetPause()->SetEnabled(kFALSE);
  Cont->GetNevent()->SetState(kTRUE);
  Cont->GetFEntry()->SetEnabled(kTRUE);
  Cont->GetSavDat()->SetEnabled(kTRUE);
  reset->SetEnabled(kTRUE);
  for(int i=1; i<=3; i++){
    Opt->GetRunMode()->GetButton(i)->SetEnabled(kTRUE);
  }

  pauseflg=0;
  //  for(int i=0; i<3; i++){
  //    if(Opt->GetRunMode()->GetButton(i)->IsOn())runmode=i;
  //  }
  for(int i=0; i<128; i++){
    for(int j=0; j<128; j++){
      raw[i][j]=0;
    }
  }
}

void MyMainFrame::PauseDAQ()
{
  if(pauseflg==0){
    sitcp->DAQStart(false);
    utime->TurnOff();
    dtime->TurnOff();

    SetStatus("Pause run",7);
    pauseflg=1;
    Cont->GetStart()->SetEnabled(kTRUE);
    Cont->GetPause()->SetEnabled(kFALSE);
  }
}

void MyMainFrame::ClearLogText()
{
  int retval;
  wrMsg = new TGMsgBox(gClient->GetRoot(),this,"Question","Do you realy clear log?",kMBIconQuestion,kMBYes | kMBNo,&retval);
  
  if(retval==1) Log->GetlogEdit()->Clear();
}

void MyMainFrame::SaveLogText()
{
  SetStatus("Save log as text file",5);
  Log->GetlogEdit()->SaveFile(0,kTRUE);
}


void MyMainFrame::DirFind()
{
  TGFileInfo fInfo;
  new TGFileDialog(fClient->GetDefaultRoot(), this, kFDOpen, &fInfo);
 
  if(fInfo.fFilename){
    char dispstr[2048];
    filedir = opendir(fInfo.fFilename);
    if(filedir==NULL){
      sprintf(dispstr, "Directory PATH=%s dose not exist", fInfo.fFilename);
      int retval;
      new TGMsgBox(gClient->GetRoot(),this,"Warnig",dispstr,kMBIconExclamation,kMBOk,&retval);
    }else{
      if(strstr(fInfo.fFilename,getenv("HOME"))==NULL){
	int retval;
	new TGMsgBox(gClient->GetRoot(),this,"Warning","You should select file in your home directory",kMBIconExclamation,kMBOk,&retval);
	fildirpath = getenv("PWD");
      }else{
	strcpy(fildirpath,fInfo.fFilename);
      }
      sprintf(dispstr, "Set saving directory path:%s",fildirpath);
      SetStatus(dispstr, 5);
      Cont->GetDEntry()->SetText(GetRltPath(fildirpath));
    }
    closedir(filedir);
  }
}


void MyMainFrame::SetDAQTime()
{
  int DAQInt_ID = Canv->GetDAQIntCBox()->GetSelected();
  sitcp->SetNIM_CLK(DAQInt_ID);
}

void MyMainFrame::SetUpdateTime()
{
  utime->SetTime( (int)(Canv->GetUpdInt()->GetNumber()*1000));
}

void MyMainFrame::SetHistRange()
{
  int Hi = (int)(Opt->GetHistHi()->GetNumber());
  int Low = (int)(Opt->GetHistLow()->GetNumber());
  if(Low<Hi){
    for(int i=0; i<7 ; i++){
      h2[i]->GetZaxis()->SetRangeUser(Low, Hi);
    }
  }else{
    Opt->GetHistHi()->SetNumber(Low);
    Opt->GetHistLow()->SetNumber(Hi);
  }
}

void MyMainFrame::SetRunMode(){
  for(int i=1; i<=3; i++){
    if(Opt->GetRunMode()->GetButton(i)->IsOn())runmode=i;
  }
  if(runmode==3)Opt->GetCalib()->SetState(kFALSE);
  else Opt->GetCalib()->SetState();
}

void MyMainFrame::SetIntTime(){
  int IntegrationTime = (int)Para->GetIntTime()->GetNumber();
  IntegrationTime = IntegrationTime*1000/40;
  sitcp->SetRSTInterbal(IntegrationTime);
}

void MyMainFrame::SetRSTSig(){
  int RSTSiglen = (int)Para->GetRSTSig()->GetNumber();
  if(beforeRSTSig+1==RSTSiglen){
    RSTSiglen += 40;
    Para->GetRSTSig()->SetNumber(beforeRSTSig+40);
    beforeRSTSig += 40;
  }else if(beforeRSTSig-1==RSTSiglen){
    RSTSiglen -= 40;
    Para->GetRSTSig()->SetNumber(beforeRSTSig-40);
    beforeRSTSig -= 40;
  }else{
    RSTSiglen = RSTSiglen/40*40;
    Para->GetRSTSig()->SetNumber(RSTSiglen);
    beforeRSTSig = RSTSiglen;
  }
  sitcp->SetRSTHiTime(RSTSiglen/40);
}

void MyMainFrame::SetPixScan(){
  int PixScan = (int)Para->GetPixScan()->GetNumber();
  if(beforePixScan+1==PixScan){
    PixScan += 40;
    Para->GetPixScan()->SetNumber(beforePixScan+40);
    beforePixScan += 40;
  }else if(beforePixScan-1==PixScan){
    PixScan -= 40;
    Para->GetPixScan()->SetNumber(beforePixScan-40);
    beforePixScan = (int)Para->GetPixScan()->GetNumber();
    //    beforePixScan -= 40;
  }else{
    PixScan = PixScan/40*40;
    Para->GetPixScan()->SetNumber(PixScan);
    beforePixScan = PixScan;
  }
  sitcp->SetPixScanTime((int)Para->GetPixScan()->GetNumber()/40);
}

void MyMainFrame::SetRSTV(){
  int RSTV = (int)Para->GetRSTV()->GetNumber();
  sitcp->DAC_cnt(0,RSTV);
}

void MyMainFrame::SetStatus(char *string, unsigned char flags)
{
   char dispstr[2048];
   int yr, mt, dt, hr, mn, sc;

   if (flags & 0x04){
     tstmp = new TTimeStamp(); 
     int a = tstmp->GetDate(kFALSE);
     yr = a/10000;
     mt = (a%10000)/100;
     dt = (a%10000)%100;
     int b = tstmp->GetTime(kFALSE);
     hr = b/10000;
     mn = (b%10000)/100;
     sc = (b%10000)%100;
     sprintf(dispstr,"[%4d-%02d-%02d %02d:%02d:%02d] %s",yr,mt,dt,hr,mn,sc,string);
   }else{
     strcpy(dispstr,string);
   }
 
   
   if (flags & 0x01){
     Log->GetlogEdit()->AddLine(dispstr) ;
     Log->GetlogEdit()->ScreenDown() ;
   }
   
   if ((flags & 0x02) && logfout){
     *logfout << dispstr << endl;
   }

   
   
}  


char *MyMainFrame::GetRltPath(char *fullpath)
{
  char scnchr[1024], pathtmp[1024];
  char *chrfm = "%s";
  char *rltvpathtmp;
  rltvpathtmp = (char *)malloc(1024);
  sprintf(scnchr, "%s/%s", getenv("HOME"), chrfm);
  if(sscanf(fullpath, scnchr, pathtmp)==1){
    sprintf(rltvpathtmp, "~/%s", pathtmp);
    return rltvpathtmp;
  }else{
    sprintf(rltvpathtmp, "~/");
    return rltvpathtmp;
  }
}

void MyMainFrame::CloseWindow()
{
  int retval;
  clMsg = new TGMsgBox(gClient->GetRoot(),this,"Question","Are you sure to quit?",kMBIconQuestion,kMBYes | kMBNo,&retval);

  if(retval == 1){
    sitcp->SetRSTInterbal(0);
    sitcp->SetNIM_CLK(0);
    sitcp->SetRSTHiTime(0);
    sitcp->SetPixScanTime(0);
    sitcp->DAC_cnt(0,0);
    sitcp->CloseUDPSock();
    sitcp->CloseTCPSock();
    gApplication->Terminate(0);  
  }
}


MyMainFrame::~MyMainFrame() 
{
  Cleanup();
}



//
//Menu bar
MyMenuBar::MyMenuBar(const TGWindow *p) : TGMenuBar(p)
{
  MenuFile = new TGPopupMenu(gClient->GetRoot());
  MenuRun = new TGPopupMenu(gClient->GetRoot());
}

void MyMenuBar::MenuLayout()
{
  MenuFile->AddEntry("&Reset all", M_FILE_ALLRESET);
  MenuFile->AddEntry("&Save", M_FILE_SAVE);
  MenuFile->AddEntry("S&ave as...", M_FILE_SAVEAS);
  MenuFile->AddSeparator();
  MenuFile->AddEntry("E&xit", M_FILE_EXIT);

  MenuRun->AddEntry("&Start", M_RUN_START);
  MenuRun->AddEntry("S&top", M_RUN_STOP);
  MenuRun->AddEntry("&Pause", M_RUN_PAUSE);
  MenuRun->AddSeparator();
  MenuRun->AddEntry("&Reset Canvas", M_RUN_RESETCANV);
 
  AddPopup("&File", MenuFile, new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0));
  AddPopup("&Run", MenuRun, new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0));
}

MyMenuBar::~MyMenuBar()
{
  Cleanup();
}

//
//Control Frame
MyCtrl::MyCtrl(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
  start = new TGTextButton(this,"&Start");
  stop = new TGTextButton(this,"&Stop");
  pause = new TGTextButton(this,"&Pause");
  start->SetWidth(150);
  stop->SetWidth(150);
  pause->SetWidth(150);

  NeveLabel = new TGLabel(this,"Number of events");
  CeveLabel = new TGLabel(this,"Current event");
  NeveEntry = new TGNumberEntry(this, 0, 10, 11,TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 0);
  CeveEntry = new TGNumberEntryField(this, 12, 0,TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 0);
  SavDat = new TGCheckButton(this, "save data",1);
  SavDirEntry = new TGTextEntry(this);
  SavFilEntry = new TGTextEntry(this);
  SavDirLabel = new TGLabel(this,"Save directory path\n(relative path)");
  SavDirSel = new TGPictureButton(this,"/usr/local/root/icons/fileopen.xpm");
  SavFilLabel = new TGLabel(this,"Save file name");

  CeveEntry->SetEnabled(kFALSE);
  SavDirEntry->SetEnabled(kFALSE);
}

void MyCtrl::CtrlLayout()
{
  AddFrame(start, new TGLayoutHints(kLHintsExpandX,30,30,5,4));
  AddFrame(stop, new TGLayoutHints(kLHintsExpandX,30,30,3,4));
  AddFrame(pause, new TGLayoutHints(kLHintsExpandX,30,30,3,4));

  AddFrame(NeveLabel, new TGLayoutHints(kLHintsLeft,35,5,30,1));//left,right,top,bottom
  AddFrame(NeveEntry, new TGLayoutHints(kLHintsExpandX,46,38,1,3));
  AddFrame(CeveLabel, new TGLayoutHints(kLHintsLeft,35,5,3,1));
  AddFrame(CeveEntry, new TGLayoutHints(kLHintsExpandX,46,51,1,3));

  AddFrame(SavDat, new TGLayoutHints(kLHintsLeft,25,5,30,1));
  AddFrame(SavDirLabel, new TGLayoutHints(kLHintsLeft,25,5,5,1));
  AddFrame(SavDirEntry, new TGLayoutHints(kLHintsExpandX,25,25,1,1));
  AddFrame(SavDirSel, new TGLayoutHints(kLHintsRight,25,25,1,1));

  AddFrame(SavFilLabel, new TGLayoutHints(kLHintsLeft,25,5,5,1));
  AddFrame(SavFilEntry, new TGLayoutHints(kLHintsExpandX,25,25,1,1));
 
}

MyCtrl::~MyCtrl()
{
  Cleanup();
}

//
//Canvas Frame
MyCanvas::MyCanvas(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h)
{ 
  ECanvas = new TRootEmbeddedCanvas("Ecanvas",this,400,400);
  
  CanvOptFrame = new TGHorizontalFrame(this,500,100);  
  TimerFrame = new TGGroupFrame(CanvOptFrame,"Timer",kHorizontalFrame);
  TimerDAQFrame = new TGHorizontalFrame(TimerFrame,200,20);
  TimerUpdFrame = new TGHorizontalFrame(TimerFrame,200,20);
  DAQIntLabel = new TGLabel(TimerDAQFrame,"DAQ freqency [Hz]");
  UpdIntLabel = new TGLabel(TimerUpdFrame,"Update interval [s]");
  UpdIntEntry = new TGNumberEntry(TimerUpdFrame, 0.1, 5, 14, TGNumberFormat::kNESRealThree, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 10);

  DAQIntCBox = new TGComboBox(TimerDAQFrame,10);

}

void MyCanvas::CanvLayout()
{
  DAQIntCBox->AddEntry("1.5", 0);
  DAQIntCBox->AddEntry("  3", 1);
  DAQIntCBox->AddEntry("  6", 2);
  DAQIntCBox->AddEntry(" 12", 3);
  DAQIntCBox->AddEntry(" 24", 4);
  DAQIntCBox->AddEntry(" 48", 5);
  DAQIntCBox->AddEntry(" 95", 6);
  DAQIntCBox->AddEntry("191", 7);
  DAQIntCBox->AddEntry("  0", 8);
  DAQIntCBox->Select(1);
  DAQIntCBox->Resize(50, 20);
  
  AddFrame(ECanvas, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 5,5,5,3));
  
  TimerDAQFrame->AddFrame(DAQIntLabel, new TGLayoutHints(kLHintsNormal,15,5,3,0));
  TimerDAQFrame->AddFrame(DAQIntCBox, new TGLayoutHints(kLHintsRight,4,5,1,0));
  TimerUpdFrame->AddFrame(UpdIntLabel, new TGLayoutHints(kLHintsNormal,15,5,3,0));
  TimerUpdFrame->AddFrame(UpdIntEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));  
  TimerFrame->AddFrame(TimerDAQFrame, new TGLayoutHints(kLHintsCenterX, 5,5,0,0));
  TimerFrame->AddFrame(TimerUpdFrame, new TGLayoutHints(kLHintsCenterX, 5,5,0,0));
  CanvOptFrame->AddFrame(TimerFrame, new TGLayoutHints(kLHintsLeft, 5,5,0,0));
 

  AddFrame(CanvOptFrame, new TGLayoutHints(kLHintsLeft, 5,5,0,1)); 

}


MyCanvas::~MyCanvas()
{
  Cleanup();
}


//
//Option Frame
MyOption::MyOption(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
  DOptFrame = new TGGroupFrame(this,"Display",kVerticalFrame);
  DOptLabel = new TGLabel(DOptFrame,"Display Style");
  DOptCBox = new TGComboBox(DOptFrame,100);
  HContLabel = new TGLabel(DOptFrame,"Histogram content");
  HContCBox = new TGComboBox(DOptFrame,100);
  LogX = new TGCheckButton(DOptFrame, "Set Log X(1D only)",1);
  LogY = new TGCheckButton(DOptFrame, "Set Log Y(1D only)",1);
  LogZ = new TGCheckButton(DOptFrame, "Set Log Z(2D only)",1);

  RunVertical = new TGVerticalFrame(this, 100, 100);
  TakePedeRadio = new TGButtonGroup(RunVertical, "Run Mode", kVerticalFrame);
  fR[0] = new TGRadioButton(TakePedeRadio, new TGHotString("&Calib Data Only"));
  fR[1] = new TGRadioButton(TakePedeRadio, new TGHotString("&Calib Data & Data"));
  fR[2] = new TGRadioButton(TakePedeRadio, new TGHotString("&Data Only"));
  CEntLabel = new TGLabel(this,"Number of Calib event");
  CalibEntry = new TGNumberEntry(this, 0, 10, 15, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELNoLimits, 0, 0);

  HistRangeGroup = new TGGroupFrame(this,"2D Hist Range",kVerticalFrame);
  HistRangeLFrame = new TGHorizontalFrame(HistRangeGroup,100,100);  
  HistRangeEFrame = new TGHorizontalFrame(HistRangeGroup,100,100);  
  RangeHiLabel = new TGLabel(HistRangeLFrame,"         Max");
  RangeLowLabel = new TGLabel(HistRangeLFrame,"Min");
  HistHiEntry = new TGNumberEntry(HistRangeEFrame, 0.1, 5, 14, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 4096);
  HistLowEntry = new TGNumberEntry(HistRangeEFrame, 0.1, 5, 14, TGNumberFormat::kNESInteger, TGNumberFormat::kNEANonNegative, TGNumberFormat::kNELLimitMinMax, 0, 4096);

}

void MyOption::OptLayout()
{
  DOptCBox->AddEntry("2D hist", 0);
  DOptCBox->AddEntry("1D hist", 1);
  DOptCBox->Select(0);
  DOptCBox->Resize(150, 20);

  HContCBox->AddEntry("Raw data", 0);
  HContCBox->AddEntry("Raw Data - Ped.", 1); 
  HContCBox->AddEntry("Raw Data - Ped.- CMS", 2); 
  HContCBox->AddEntry("SNR", 3); 
  HContCBox->AddEntry("Pedestal", 4);
  HContCBox->AddEntry("Noise", 5);
  HContCBox->AddEntry("Bad channel", 6);
  HContCBox->Select(0);
  HContCBox->Resize(150, 20);  

  DOptFrame->AddFrame(DOptLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  DOptFrame->AddFrame(DOptCBox, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
  DOptFrame->AddFrame(HContLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  DOptFrame->AddFrame(HContCBox, new TGLayoutHints(kLHintsExpandX,5,5,3,5));
  DOptFrame->AddFrame(LogX, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  DOptFrame->AddFrame(LogY, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  DOptFrame->AddFrame(LogZ, new TGLayoutHints(kLHintsLeft,5,5,3,5));

  HistRangeLFrame->AddFrame(RangeHiLabel, new TGLayoutHints(kLHintsRight,4,3,0,0));  
  HistRangeLFrame->AddFrame(RangeLowLabel, new TGLayoutHints(kLHintsRight,4,3,0,0));  
  HistRangeEFrame->AddFrame(HistHiEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));  
  HistRangeEFrame->AddFrame(HistLowEntry, new TGLayoutHints(kLHintsRight,4,5,1,0));  
  
  HistRangeGroup->AddFrame(HistRangeLFrame, new TGLayoutHints(kLHintsLeft,5,5,0,0));
  HistRangeGroup->AddFrame(HistRangeEFrame, new TGLayoutHints(kLHintsLeft,5,5,0,0));
 
  fR[1]->SetState(kButtonDown);
  TakePedeRadio->Show();
  RunVertical->AddFrame(TakePedeRadio, new TGLayoutHints(kLHintsLeft,5,5,0,0));

  AddFrame(DOptFrame, new TGLayoutHints(kLHintsExpandX,5,5,0,1));
  AddFrame(HistRangeGroup, new TGLayoutHints(kLHintsExpandX,5,5,0,1));
  AddFrame(RunVertical, new TGLayoutHints(kLHintsExpandX,5,5,0,0));
  AddFrame(CEntLabel, new TGLayoutHints(kLHintsCenterX,5,5,0,1));
  AddFrame(CalibEntry, new TGLayoutHints(kLHintsRight,5,20,0,1));

}

MyOption::~MyOption()
{
  Cleanup();
}

//
//INTPIX Parameters setting
MyParameters::MyParameters(const TGWindow* p, UInt_t w, UInt_t h) : TGVerticalFrame(p,w,h,kFixedWidth)
{
  INTPIXParaFrame = new TGGroupFrame(this,"INTPIX Parameters",kVerticalFrame);
  IntTimeLabel = new TGLabel(INTPIXParaFrame,"Integration Time [us]");
  IntTimeEntry = new TGNumberEntry(INTPIXParaFrame, 0.1, 5, 14,
				   TGNumberFormat::kNESInteger,
				   TGNumberFormat::kNEANonNegative,
				   TGNumberFormat::kNELLimitMinMax,
				   300*40/1000, 6000000*40/1000);
  RSTSigLabel = new TGLabel(INTPIXParaFrame,"RST Signal Length [ns]");
  RSTSigEntry = new TGNumberEntry(INTPIXParaFrame, 0.1, 5, 14,
				   TGNumberFormat::kNESInteger,
				   TGNumberFormat::kNEANonNegative,
				   TGNumberFormat::kNELLimitMinMax,
				   40, 255*40);
  PixScanLabel = new TGLabel(INTPIXParaFrame,"Scan Time [ns/pixel]");
  PixScanEntry = new TGNumberEntry(INTPIXParaFrame, 0.1, 5, 14,
				   TGNumberFormat::kNESInteger,
				   TGNumberFormat::kNEANonNegative,
				   TGNumberFormat::kNELLimitMinMax,
				   40, 16*40);
  RSTVLabel = new TGLabel(INTPIXParaFrame,"RSTV Voltage[mV]");
  RSTVEntry = new TGNumberEntry(INTPIXParaFrame, 0.1, 5, 14,
				   TGNumberFormat::kNESInteger,
				   TGNumberFormat::kNEANonNegative,
				   TGNumberFormat::kNELLimitMinMax,
				   0, 1800);
  
}

void MyParameters::ParaLayout()
{
  INTPIXParaFrame->AddFrame(IntTimeLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  INTPIXParaFrame->AddFrame(IntTimeEntry, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
  INTPIXParaFrame->AddFrame(RSTSigLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  INTPIXParaFrame->AddFrame(RSTSigEntry, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
  INTPIXParaFrame->AddFrame(PixScanLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  INTPIXParaFrame->AddFrame(PixScanEntry, new TGLayoutHints(kLHintsExpandX,5,5,3,1));
  INTPIXParaFrame->AddFrame(RSTVLabel, new TGLayoutHints(kLHintsLeft,5,5,3,1));
  INTPIXParaFrame->AddFrame(RSTVEntry, new TGLayoutHints(kLHintsExpandX,5,5,3,1));

  AddFrame(INTPIXParaFrame, new TGLayoutHints(kLHintsExpandX,5,5,0,1));
}

MyParameters::~MyParameters()
{
  Cleanup();
}
  
//log editor
MylogEdit::MylogEdit(const TGWindow* p, TGString* title) : TGGroupFrame(p,title,kHorizontalFrame)
{
  LogCtrlFrame = new TGVerticalFrame(this, 50, 100);
  LogEdit = new TGTextEdit(this, 600, 120);
  clearlog = new TGTextButton(LogCtrlFrame, "&Clear Log");
  savelog = new TGTextButton(LogCtrlFrame, "Save &Log");
}

void MylogEdit::EdtLayout()
{
  LogCtrlFrame->AddFrame(savelog, new TGLayoutHints(kLHintsCenterX | kLHintsBottom,2,2,3,3));
  LogCtrlFrame->AddFrame(clearlog, new TGLayoutHints(kLHintsCenterX | kLHintsBottom,2,2,3,3));

  AddFrame(LogEdit, new TGLayoutHints(kLHintsExpandX | kLHintsExpandY,2,2,3,3));  
  AddFrame(LogCtrlFrame, new TGLayoutHints(kLHintsRight | kLHintsBottom,2,2,3,3));

  LogEdit->EnableMenu(kFALSE);
  LogEdit->SetReadOnly(kTRUE); 
}

MylogEdit::~MylogEdit()
{
  Cleanup();
}

